<?php

/**
 * LeadsTasksPresetMapper.php
 *
 *
 * @method Leads_Mapper_LeadsTasksPresetMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsTasksPresetMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsTasksPresetModel';

    protected $_dbTable = 'Leads_DbTable_LeadsTasksPresetDbTable';

    /**
     * Save lead tasks preset model to DB
     * @param $model Leads_Model_LeadsTasksPresetModel
     * @return Leads_Model_LeadsTasksPresetModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'preset_name' => $model->getPresetName(),
            'task_title' => $model->getTaskTitle(),
            'task_note' => $model->getTaskNote(),
            'created_at' => $model->getCreatedAt(),
            'creator_id' => $model->getCreatorId(),
            'updated_at' => $model->getUpdatedAt(),
            'editor_id' => $model->getEditorId()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get presets
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = false
    ) {

        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $createdAt = 'pltp.created_at';

        if ($withTimezoneOffset === true) {
            $createdAt = new Zend_Db_Expr("CONVERT_TZ(`pltp`.`created_at`, '+00:00', '$mysqlOffset')");
        }

        $updatedAt = 'pltp.updated_at';

        if ($withTimezoneOffset === true) {
            $updatedAt = new Zend_Db_Expr("CONVERT_TZ(`pltp`.`updated_at`, '+00:00', '$mysqlOffset')");
        }

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pltp' => 'plugin_leads_tasks_presets'),
                array(
                    'pltp.id',
                    'pltp.preset_name',
                    'pltp.task_title',
                    'pltp.task_note',
                    'pltp.creator_id',
                    'createdAt' => $createdAt,
                    'pltp.editor_id',
                    'updatedAt' => $updatedAt,
                    'creatorName' => 'u.full_name',
                    'editorName' => 'uc.full_name'
                )
            )
            ->joinLeft(array('u' => 'user'), 'u.id=pltp.creator_id', array())
            ->joinLeft(array('uc' => 'user'), 'uc.id=pltp.editor_id', array());


        $select->group('pltp.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(pltp.id))'));

            $select->from(array('pltp' => 'plugin_leads_tasks_presets'), $count)
                ->joinLeft(array('u' => 'user'), 'u.id=pltp.creator_id', array())
                ->joinLeft(array('uc' => 'user'), 'uc.id=pltp.editor_id', array());


            $select = $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            if (empty($count['count'])) {
                $count['count'] = 0;
            }

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get associative array of task presets
     *
     * @param bool $pairs select pairs flag
     * @return array
     */
    public function getPresetsData($pairs = false)
    {
        $select = $this->getDbTable()->getAdapter()->select()->from(array('pltp' => 'plugin_leads_tasks_presets'),
            array('id', 'preset_name', 'task_title', 'task_note'));
        if ($pairs === true) {
            $select = $this->getDbTable()->getAdapter()->select()->from(array('pltp' => 'plugin_leads_tasks_presets'),
                array('id', 'preset_name'));
            return $this->getDbTable()->getAdapter()->fetchPairs($select);
        }
        return $this->getDbTable()->getAdapter()->fetchAssoc($select);

    }

    /**
     * Find by preset name
     *
     * @param string $presetName preset name
     * @return null
     */
    public function findByPresetName($presetName)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('preset_name = ?', $presetName);
        return $this->_findWhere($where);
    }

    /**
     * Delete form task presets config record
     *
     * @param int $id config id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_tasks_presets', $where);

    }

}
