<?php

/**
 * LeadsWebhookUrlsMapper.php
 *
 *
 * @method Leads_Mapper_LeadsWebhookUrlsMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Leads_Mapper_LeadsWebhookUrlsMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Leads_Model_LeadsWebhookUrlsModel';

    protected $_dbTable = 'Leads_DbTable_LeadsWebhookUrlsDbTable';

    /**
     * Save lead webhook URL's model to DB
     * @param $model Leads_Model_LeadsWebhookUrlsModel
     * @return Leads_Model_LeadsWebhookUrlsModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'webhook_name'        => $model->getWebhookName(),
            'webhook_type'        => $model->getWebhookType(),
            'webhook_action_type' => $model->getWebhookActionType(),
            'action_url'          => $model->getActionUrl(),
            'created_at'          => $model->getCreatedAt(),
            'creator_id'          => $model->getCreatorId(),
            'updated_at'          => $model->getUpdatedAt(),
            'editor_id'           => $model->getEditorId()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     * Get configs
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = false
    ) {

        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $createdAt = 'plwu.created_at';

        if ($withTimezoneOffset === true) {
            $createdAt = new Zend_Db_Expr("CONVERT_TZ(`plwu`.`created_at`, '+00:00', '$mysqlOffset')");
        }

        $updatedAt = 'plwu.updated_at';

        if ($withTimezoneOffset === true) {
            $updatedAt = new Zend_Db_Expr("CONVERT_TZ(`plwu`.`updated_at`, '+00:00', '$mysqlOffset')");
        }

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plwu' => 'plugin_leads_webhook_urls'),
                array(
                    'plwu.id',
                    'plwu.webhook_name',
                    'plwu.webhook_type',
                    'plwu.webhook_action_type',
                    'plwu.action_url',
                    'plwu.creator_id',
                    'createdAt' => $createdAt,
                    'plwu.editor_id',
                    'updatedAt' => $updatedAt
                )
            )
            ->joinLeft(array('u' => 'user'), 'u.id=plwu.creator_id', array())
            ->joinLeft(array('uc' => 'user'), 'uc.id=plwu.editor_id', array());


        $select->group('plwu.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(plwu.id))'));

            $select->from(array('plwu' => 'plugin_leads_webhook_urls'), $count)
                ->joinLeft(array('u' => 'user'), 'u.id=plwu.creator_id', array())
                ->joinLeft(array('uc' => 'user'), 'uc.id=plwu.editor_id', array());


            $select = $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            if (empty($count['count'])) {
                $count['count'] = 0;
            }

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get associative array of webhook configs
     *
     * @param string $webHookType select pairs flag
     * @param string $webhookActionType web
     * @return array
     */
    public function getPresetsData($webHookType = '', $webhookActionType = '')
    {
        $select = $this->getDbTable()->getAdapter()->select()->from(array('plwu' => 'plugin_leads_webhook_urls'),
            array(
                'id',
                'webhook_name',
                'webhook_type',
                'webhook_action_type',
                'action_url',
            ));

        $where = '';

        if (!empty($webHookType)) {
            if (!empty($where)) {
                $where .= ' AND ';
            }

            $where .= $this->getDbTable()->getAdapter()->quoteInto('webhook_type = ?', $webHookType);
        }

        if (!empty($webhookActionType)) {
            if (!empty($where)) {
                $where .= ' AND ';
            }

            $where .= $this->getDbTable()->getAdapter()->quoteInto('webhook_action_type = ?', $webhookActionType);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Find by config name
     *
     * @param string $configName preset name
     * @return null
     */
    public function findByConfigName($configName)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('webhook_name = ?', $configName);
        return $this->_findWhere($where);
    }

    /**
     * Delete form webhook urls config record
     *
     * @param int $id config id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_leads_webhook_urls', $where);

    }

}
